/*
 * Copyright (C) 2001 the MCatS project
 *
 * This file is part of the MCatS, Modular Catalog System.
 *
 * MCatS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MCatS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 */

#ifndef __MCATS_H_
#define __MCATS_H_

#include <sys/param.h>
#include <sys/stat.h>
#include <sys/mount.h>

#define MCATS_DIR ".mcats"
#define MCATS_PLUGIN_DIR ".mcats/plugins"
#define SYSTEM_PLUGIN_DIR PLUGINS_DIR
#define CONFIG_FILE_NAME "mcats.conf"

#define WRONG_MOUDLE_NAME 1
#define ERROR_OPENING_MODULE 2

/* defined for different module types */
#define MODULE_INPUT 1
#define MODULE_OUTPUT 2

#ifndef LINUX
  #define NULL 0
#endif

#ifdef LINUX
  #define DL_LAZY RTLD_LAZY
  #define MFSNAMELEN 255
  #define MNAMELEN 255
#endif

/*
 * Usage of bit tech on type is the best way i think.
 * let say u have a plugin that are for gui, input and output.
 * Just set all 3 bits then, 1+2+4.. can it be easier?
 */

struct MainStruct;
struct ModuleStruct;
struct FileStruct;

/* typedef struct {
  char name[50];
  char description[255];
  int type;
} module; */

struct MainStruct {
	int argc;
	char **argv;			/* Arguments sent to MCatS by commandline */
	int input_argc;
	char **input_argv;	/* Optional input args supplied by the module */
	int output_argc;
	char **output_argv;		/* Optional output args supplied by the module */
	int (*load_config)(void);		/* Pointer to load_config function */
	void (*do_export)(struct ModuleStruct *);		/* Pointer to export function */
	void (*do_import)(struct ModuleStruct *);		/* Pointer to import function */
};

struct FileStruct {
	char disk[50];
	char filename[250];
	char path[255];
	long int size;
	int type;			/* Type, file, link etc. */
	unsigned int mode;		/* ex 750, rwxr-x--- */
	unsigned int owner;			/* ex 0, root */
	unsigned int group;			/* ex 0, wheel */
	char mime[48];	/* Mime extension */
	char *description;	/* Extension put in by a filter module */
	time_t st_atimespec;  /* time of last access */
	time_t st_mtimespec;  /* time of last data modification */
	time_t st_ctimespec;  /* time of last file status change */
	int undef1;			/* Variable for future usage */
	int undef2;			/* Variable for future usage */
	struct FileStruct *next;		/* Pointer to next FileStruct */
};

struct VolumeStruct {
	char name[33];		/* name of the media */
	char publisher[129];	/* Name of the Publisher, if any */
	char preparer[129];	/* Name of the Preparer, if any */
	char application[129];	/* Name of the Application, if any */
	/* Copied from statfs struct */
	short   f_type;     /* type of file system (unused; zero) */
        u_short f_oflags;   /* deprecated copy of mount flags */
        long    f_bsize;    /* fundamental file system block size */
        long    f_iosize;   /* optimal transfer block size */
        long    f_blocks;   /* total data blocks in file system */
        long    f_bfree;    /* free blocks in fs */
        long    f_bavail;   /* free blocks avail to non-superuser */
        long    f_files;    /* total file nodes in file system */
        long    f_ffree;    /* free file nodes in fs */
        fsid_t  f_fsid;     /* file system id */
        uid_t   f_owner;    /* user that mounted the file system */
        long    f_flags;    /* copy of mount flags */
        long    f_syncwrites; /* count of sync writes since mount */
        long    f_asyncwrites; /* count of async writes since mount */
        long    f_spare[1]; /* spare for later */
        char    f_fstypename[MFSNAMELEN]; /* fs type name */
        char    f_mntonname[MNAMELEN];    /* directory on which mounted */
        char    f_mntfromname[MNAMELEN];  /* mounted file system */

	struct FileStruct *filestructure;	/* Pointer to first FileStruct */
};

struct ModuleStruct {
	char filename[255];				/* Full path and filename to module */
	char shortname[15];				/* Name to be used for usage with -i and -o */
	char name[50];						/* Name in the plug-ins list */
	char description[255];		/* A description about the module */
	int id;										/* Identifyer of what kind of module it is \
														 * 1 input, 2 output, 4 search, 8 filter \
														 * 16 gui
														 */
	int (*output)(struct MainStruct *, struct VolumeStruct );		/* Pointer to output function */
	struct VolumeStruct (*input)(struct MainStruct *);					/* Pointer to input function */
	void (*gui)(struct MainStruct *);									/* Pointer to GUI function */
	int (*check_input_args)(int argc, char *argv[]);				/* Goes through the argv and \
																						 * see that everything is ok
																						 */
	int (*check_output_args)(int argc, char *argv[]);				/* Goes through the argv and \
																						 * see that everything is ok
																						 */
	void (*undef1)(void);											/* Function for future usage */
	void (*undef2)(void);											/* Function for future usage */
	struct ModuleStruct *next;												/* Pointer to next ModuleStruct, \
																						 * if none, NULL
																						 */
};

typedef struct {
	char shortname[15];				/* Name to be used for usage with -i and -o */
	char name[50];						/* Name in the plug-ins list */
	char description[255];		/* A description about the module */
	int id;										/* Identifyer of what kind of module it is \
														 * 1 input, 2 output, 4 search, 8 filter \
														 * 16 gui
														 */
	int (*output)(struct MainStruct *, struct VolumeStruct );		/* Pointer to output function */
	struct VolumeStruct (*input)(struct MainStruct *);					/* Pointer to input function */
	void (*gui)(struct MainStruct);									/* Pointer to GUI function */
	int (*check_input_args)(int argc, char *argv[]);				/* Goes through the argv and \
																						 * see that everything is ok
																						 */
	int (*check_output_args)(int argc, char *argv[]);				/* Goes through the argv and \
																						 * see that everything is ok
																						 */
	void (*undef1)(void);											/* Function for future usage */
	void (*undef2)(void);											/* Function for future usage */
} PluginStruct;

#endif
