/*
 * Copyright (C) 2001 the MCatS project
 *
 * This file is part of the MCatS, Modular Catalog System.
 *
 * MCatS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MCatS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <errno.h>
#include <getopt.h>
#include <malloc.h>
#include "mcats.h"

#define SHORT_NAME "stdout"
#define STDOUT_ARGS_HELP 1
#define STDOUT_ARGS_FILEFORMAT 2
#define STDOUT_ARGS_VOLUMEFORMAT 4
#define STDOUT_ARGS_VERSION 8

struct StdOutArgs {
	char fileformat[256];
	char volumeformat[256];
	int Options;
};

void usage()
{
	printf( "usage: mcats -o stdout [options]\n"
		"options:\n"
		"   --volumeformat [format]  Which fields to print to stdout\n"
		"   --fileformat [format]    Which fields to print to stdout\n"
		"   --version                Prints stdout version and exit\n"
		"   --help                   Prints this help and exit\n\n");
}

void version()
{
	printf( "stdout version %s\n", STDOUT_VERSION );
}

int
print_stdout ( struct MainStruct *mainstructure,
								struct VolumeStruct volumestructure )
{
	struct StdOutArgs stdout_args;
	int ok_args = 0;

	ok_args = read_args( mainstructure->argc, mainstructure->argv, &stdout_args );

	if( stdout_args.Options & STDOUT_ARGS_VOLUMEFORMAT )
		print_volumeformat( mainstructure, volumestructure, stdout_args );

	if( stdout_args.Options & STDOUT_ARGS_FILEFORMAT )
		print_fileformat( mainstructure, volumestructure, stdout_args );
}

int
print_volumeformat( struct MainStruct *mainstructure,
										struct VolumeStruct volumestructure,
										struct StdOutArgs stdout_args )
{
	char *toc;		/* Variable to store the structure of the output */
	char **content;	/* contain the text to be printed inbet fileinfo */
	char *stringp;	/* Pointer to original input string */
	char *seps;		/* Pointer to seperated string */
	char prevchar = 0;	/* Previous character in inputstring */
	char nextchar = 0;	/* Next character in inputstring */
	int fields = 0;	/* Number of fields in inputstring */
	int max_field_length = 0;
	int previndex = 0;
	int loop = 0;
	int ch = 0;		/* Tmp var to store current character */
	int index = 0;	/* Index when processing the inputstring */


	stringp = strdup( stdout_args.volumeformat );
	while( seps = strsep( &stringp, "%\\" ) )
	{
		if( stringp != 0 )
		{
			nextchar = *(stringp++);
			fields++;
		}
		else
			nextchar = 0;

		if( strlen( seps ) > max_field_length )
			max_field_length = strlen( seps );
		if( strlen( seps ) != 0 )
			fields++;
	}

	toc = (char *) malloc( fields );
	content = (char **)calloc( fields, max_field_length );

	for( loop = 0; loop < fields; loop++ )
 	content[loop] = (char *) malloc( max_field_length + 1 );

	stringp = strdup( stdout_args.volumeformat );
	while( seps = strsep( &stringp, "%\\" ) )
	{
		if( stringp != 0 )
		{
			previndex = strlen( stdout_args.volumeformat ) - strlen( stringp ) - 1;
			/* Get the previous character, % or \ */
			prevchar = stdout_args.volumeformat[previndex];
			nextchar = *(stringp++);
		}
		else
			prevchar = nextchar = 0;
	
		if( strlen( seps ) != 0 )
		{
			toc[index++] = 's';
			sprintf( content[index - 1], "%s", seps );
		}
		if( nextchar != 0 )
		{
			if( prevchar == '%' )
			{
				toc[index++] = 'c';
				sprintf( content[index - 1], "%c", nextchar );
			} else if( prevchar == '\\' )
			{
				toc[index++] = 'e';
				sprintf( content[index - 1], "%c", nextchar );
			}
		}
	}

	toc[index] = '\0';
		

	/* Print out information requested by the user */
	for( loop = 0; loop < strlen( toc ); loop++ )
	{
		if( toc[loop] == 'c' )
			ch = content[loop][0];
		else if( toc[loop] == 'e' )
			ch = toupper( content[loop][0] );
		else if( toc[loop] == 's' )
		  ch = 0;

		switch( ch ) {
		case 'a':
			/* Print the volumename to stdout */
			printf( "%s", volumestructure.name );
			break;
		case 'b':
			/* Print the volumepublisher to stdout */
			printf( "%s", volumestructure.publisher );
			break;
		case 'c':
			/* Print the volumepreparer to stdout */
			printf( "%s", volumestructure.preparer );
			break;
		case 'd':
			/* Print the volumeapplication to stdout */
			printf( "%s", volumestructure.application );
			break;
		case 'e':
			/* Print the type of filesystem to stdout */
			printf( "%ld", volumestructure.f_type );
			break;
		case 'f':
			/* Print copy of mount flags to stdout (deprecated) */
			printf( "%ld", volumestructure.f_oflags );
			break;
		case 'g':
			/* Print fundamental file system block size to stdout */
			printf( "%l", volumestructure.f_bsize );
			break;
		case 'h':
			/* Print optimal transfer block size to stdout */
			printf( "%l", volumestructure.f_iosize );
			break;
		case 'i':
			/* Print total data blocks in file system to stdout */
			printf( "%l", volumestructure.f_blocks );
			break;
		case 'j':
			/* Print free blocks in fs to stdout */
			printf( "%l", volumestructure.f_bfree );
			break;
		case 'k':
			/* Print free blocks avail to non-superuser to stdout */
			printf( "%l", volumestructure.f_bavail );
			break;
		case 'l':
			/* Print total file nodes in file system to stdout */
			printf( "%l", volumestructure.f_files );
			break;
		case 'm':
			/* Print free file nodes in fs to stdout */
			printf( "%l", volumestructure.f_ffree );
			break;
		case 'n':
			/* Print file system id to stdout */
			printf( "%l", volumestructure.f_fsid );
			break;
		case 'o':
			/* Print user that mounted the file system to stdout */
			printf( "%l", volumestructure.f_owner );
			break;
		case 'p':
			/* Print copy of mount flags to stdout */
			printf( "%l", volumestructure.f_flags );
			break;
		case 'q':
			/* Print count of sync writes since mount to stdout */
			printf( "%l", volumestructure.f_syncwrites );
			break;
		case 'r':
			/* Print count of async writes since mount to stdout */
			printf( "%l", volumestructure.f_asyncwrites );
			break;
		case 't':
			/* Print fs type name to stdout */
			printf( "%s", volumestructure.f_fstypename );
			break;
		case 'u':
			/* Print directory on which mounted to stdout */
			printf( "%s", volumestructure.f_mntonname );
			break;
		case 'v':
			/* Print mounted file system to stdout */
			printf( "%s", volumestructure.f_mntfromname );
			break;
		case 0:
			/* Prints userdefined string to stdout */
			printf( "%s", content[loop] );
			break;
		/* Take care of \ inputs */
		case 'N':
			/* Prints a newline */
			printf( "\n" );
			break;
		case 'T':
			/* Prints a tab */
			printf( "\t" );
			break;
		case '\\':
			/* Prints a backslash */
			printf( "\\" );
			break; 
		default:
			printf( "%c", content[loop] );
			break;
		}
	}

	printf( "\n" );

	/* Clear memory allocated for format string */
	for( loop = 0; loop < fields; loop++)
		free( content[loop] );

	free(content);

}
	
int
print_fileformat( struct MainStruct *mainstructure,
									struct VolumeStruct volumestructure,
									struct StdOutArgs stdout_args )
{
	struct FileStruct *first_file_in_struct = volumestructure.filestructure;
	struct FileStruct *current_file_in_struct;
	struct FileStruct *next_file_in_struct;
	char *toc;		/* Variable to store the structure of the output */
	char **content;	/* contain the text to be printed inbet fileinfo */
	char *stringp;	/* Pointer to original input string */
	char *seps;		/* Pointer to seperated string */
	char prevchar = 0;	/* Previous character in inputstring */
	char nextchar = 0;	/* Next character in inputstring */
	int fields = 0;	/* Number of fields in inputstring */
	int max_field_length = 0;
	int previndex = 0;
	int loop = 0;
	int ch = 0;		/* Tmp var to store current character */
	int index = 0;	/* Index when processing the inputstring */


	stringp = strdup( stdout_args.fileformat );
	while( seps = strsep( &stringp, "%\\" ) )
	{
		if( stringp != 0 )
		{
			nextchar = *(stringp++);
			fields++;
		}
		else
			nextchar = 0;

		if( strlen( seps ) > max_field_length )
			max_field_length = strlen( seps );
		if( strlen( seps ) != 0 )
			fields++;
	}

	toc = (char *) malloc( fields );
	content = (char **)calloc( fields, max_field_length );

	for( loop = 0; loop < fields; loop++ )
 	content[loop] = (char *) malloc( max_field_length + 1 );

	stringp = strdup( stdout_args.fileformat );
	while( seps = strsep( &stringp, "%\\" ) )
	{
		if( stringp != 0 )
		{
			previndex = strlen( stdout_args.fileformat ) - strlen( stringp ) - 1;
			/* Get the previous character, % or \ */
			prevchar = stdout_args.fileformat[previndex];
			nextchar = *(stringp++);
		}
		else
			prevchar = nextchar = 0;

		if( strlen( seps ) != 0 )
		{
			toc[index++] = 's';
			sprintf( content[index - 1], "%s", seps );
		}

		if( nextchar != 0 )
		{
			if( prevchar == '%' )
			{
				toc[index++] = 'c';
				sprintf( content[index - 1], "%c", nextchar );
			} else if( prevchar == '\\' )
			{
				toc[index++] = 'e';
				sprintf( content[index - 1], "%c", nextchar );
			}
		}
	}

	toc[index] = '\0';
		

	/* Print out information requested by the user */
	current_file_in_struct = first_file_in_struct;
	while( current_file_in_struct->next != 0 )
	{
		for( loop = 0; loop < strlen( toc ); loop++ )
		{
			if( toc[loop] == 'c' )
				ch = content[loop][0];
			else if( toc[loop] == 'e' )
				ch = toupper( content[loop][0] );
			else if( toc[loop] == 's' )
				ch = 0;
			
			switch( ch ) {
				case 'f':
					/* Print the filename to stdout */
					printf( "%s", current_file_in_struct->filename );
					break;
				case 'p':
					/* Print the path to stdout */
					printf( "%s", current_file_in_struct->path );
					break;
				case 's':
					/* Print the size to stdout */
					printf( "%ld", current_file_in_struct->size );
					break;
				case 't':
					/* Print the type to stdout */
					printf( "%d", current_file_in_struct->type );
					break;
				case 'm':
					/* Print the mode to stdout */
					printf( "%u", current_file_in_struct->mode );
					break;
				case 'o':
					/* Print the owner to stdout */
					printf( "%u", current_file_in_struct->owner );
					break;
				case 'g':
					/* Print the group to stdout */
					printf( "%u", current_file_in_struct->group );
					break;
				case 'i':
					/* Print the mime to stdout */
					printf( "%s", current_file_in_struct->mime );
					break;
				case 'd':
					/* Print the description to stdout */
					if( current_file_in_struct->description != NULL )
						printf( "%s", current_file_in_struct->description );
					break;
				case 0:
					/* Prints userdefined string to stdout */
					printf( "%s", content[loop] );
					break;
				/* Take care of \ inputs */
				case 'N':
					/* Prints a newline */
					printf( "\n" );
					break;
				case 'T':
					/* Prints a tab */
					printf( "\t" );
					break;
				case '\\':
					/* Prints a backslash */
					printf( "\\" );
					break;
				default:
					/* Print just the character */
					printf( "%c", content[loop][0] );
					break;
			}
		}

		printf( "\n" );
		current_file_in_struct = current_file_in_struct->next;
	}

	/* Clear memory allocated for format string */
	for( loop = 0; loop < fields; loop++)
		free( content[loop] );

	free(content);

	current_file_in_struct = first_file_in_struct;
	while( current_file_in_struct )
	{
		next_file_in_struct = current_file_in_struct->next;
		if( current_file_in_struct->description )
			free( current_file_in_struct->description );
		free( current_file_in_struct );
		current_file_in_struct = next_file_in_struct;
	}
}

int
read_args( int argc, char *argv[], struct StdOutArgs *stdout_args )
{
	int c;
	int stop_reading_args = 0;
	int loop = 0;
	/* Set getopt's index pointer */
	while( strcmp( argv[loop++], "-o" ) != 0 )
		;
	optind = loop - 1;


/* Clear the structure so we can se if it was filled */
	sprintf( stdout_args->fileformat, "" );
	sprintf( stdout_args->volumeformat, "" );
	stdout_args->Options = 0;


	while(1)
	{
		int option_index = 0;

		static struct option stdout_options[] =
		{
			{"output", 1, 0, 0 },
			{"fileformat", 1, 0, 1},
			{"volumeformat", 1, 0, 2},
			{"version", 0, 0, 3},
			{"help", 0, 0, 4},
			{"input", 1, 0, 5},
			{0, 0, 0, 0}
		};

		c = getopt_long( argc, argv, "o:i:", stdout_options, &option_index );

		if( c == -1 )
			break;

		switch( c )
		{
			case 'o':
			case 0:
				if( strcmp( optarg, SHORT_NAME ) == 0 )
					stop_reading_args = 0;
				else
					stop_reading_args = 1;
				break;
			case 1:
				strncpy( stdout_args->fileformat, optarg, 255 );
				stdout_args->Options |= STDOUT_ARGS_FILEFORMAT;
				break;
			case 2:
				strncpy( stdout_args->volumeformat, optarg, 255 );
				stdout_args->Options |= STDOUT_ARGS_VOLUMEFORMAT;
				break;
			case 3:
				stdout_args->Options |= STDOUT_ARGS_VERSION;
				break;
			case 4:
				stdout_args->Options |= STDOUT_ARGS_HELP;
				break;
			case 'i':
			case 5:
				stop_reading_args = 1;
				break;
			default:
				break;
		}

		if( stop_reading_args == 1 )
			break;
	}

	return 0;
}


int
check_args( int argc, char *argv[] )
{
	struct StdOutArgs stdout_args;
	int ok_args = 0;

	ok_args = read_args( argc, argv, &stdout_args );

	/**
	 * If the user typo the read_args function returns
	 * -1. This will print out an usage prompt to the user
	 */
	if( ok_args == -1 )
		return -1;

	/* Do the user want the version? */
	if( stdout_args.Options & STDOUT_ARGS_VERSION )
	{
		version();
		return -1;
	}

	/* Do the user want usage info? */
	if( stdout_args.Options & STDOUT_ARGS_HELP )
	{
		usage();
		return -1;
	}

	/* Check for required arguments */
	if( !( stdout_args.Options & STDOUT_ARGS_VOLUMEFORMAT )
			&& !( stdout_args.Options & STDOUT_ARGS_FILEFORMAT ) )
	{
		usage();
		return -1;
	}

	return 0;
}

static PluginStruct StdOutStructure = 
{
	SHORT_NAME,
	"Standard Output",
	"Output to Standard Output",
	2,
	print_stdout,
	NULL,
	NULL,
	NULL,
	check_args,
	NULL,
	NULL
};

PluginStruct *
load_plugin()
{
	return &StdOutStructure;
}
