/* 
 * Copyright (C) 2001 the MCatS project
 * 
 * This file is part of the MCatS, Modular Catalog System.
 *
 * MCatS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MCatS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 */

#ifndef __WII_H_
#define __WII_H_

struct wii300header {
	char *catalogname;
	char *diskname;
	char *description;
	char medianame[255];
	char fsname[255];
	unsigned int totaldisks;
	unsigned int disknumbers[512];
	unsigned int format;
	unsigned int disknumber;
	unsigned int bytespersector;
	unsigned int sectorspercluster;
	unsigned int maxfilelength;
	unsigned int totalfiles;
	unsigned int totaldirs;
	unsigned int totalclusters;
	unsigned int freeclusters;
	unsigned int type;
	unsigned int serialnumber[4];
};

struct wii300file {
	char filename[255];
	char *filedescription;
	time_t timestamp;
	unsigned int filetype;
	unsigned int desc;
	long filesize;
};

struct wii300dir {
	char *dirname;
	char *description;
	time_t timestamp;
	unsigned int placeintree;
	unsigned int dirtype;
	unsigned int desc;
	long size;
	unsigned int files;
};


int check300version( FILE * );
void wii300_skippastcatalogname( FILE * );
void wii300_skippastdiskname( FILE * ); 
void wii300_skippastmedianame( FILE * );
void wii300_skippastfsname( FILE * ); 

void wii300_seektocatalogname( FILE *, struct wii300header * );
void wii300_seektodiskname( FILE * );
void wii300_seektomedianame( FILE * );
void wii300_seektofsname( FILE * );

int wii300_finddisk( FILE *, struct wii300header *, int );
struct wii300header * wii300_readtopheader( FILE * );
void wii300_readheader( FILE *, struct wii300header * );
void wii300_readtotaldisks( FILE *, struct wii300header * );
void wii300_readdisknumbers( FILE *, struct wii300header * );
void wii300_readcatalogname( FILE *, struct wii300header * );
void wii300_readdiskname( FILE *, struct wii300header * );
void wii300_readheaderformat( FILE *, struct wii300header * );
void wii300_readdisknumber( FILE *, struct wii300header * );
void wii300_readbytespersector( FILE *, struct wii300header * );
void wii300_readsectorspercluster( FILE *, struct wii300header * );
void wii300_readmaxfilelength( FILE *, struct wii300header * );
void wii300_readtotalfiles( FILE *, struct wii300header * );
void wii300_readtotaldirs( FILE *, struct wii300header * );
void wii300_readtotalclusters( FILE *, struct wii300header * );
void wii300_readfreeclusters( FILE *, struct wii300header * );
void wii300_readheadertype( FILE *, struct wii300header * );
void wii300_readserialnumber( FILE *, struct wii300header * );
void wii300_readheaderdescription( FILE *, struct wii300header * );
void wii300_readmedianame( FILE *, struct wii300header * );
void wii300_readfsname( FILE *, struct wii300header * );

void wii300_freeheader( struct wii300header * );

struct wii300file * wii300_readfile( FILE * );
void wii300_readfiletype( FILE *, struct wii300file * );
void wii300_readfilename( FILE *, struct wii300file * );
void wii300_readfilecreationdate( FILE *, struct wii300file * );
void wii300_readfilesize( FILE *, struct wii300file * );
void wii300_readfiledescription( FILE *, struct wii300file * );
void wii300_freefile( struct wii300file * );

struct wii300dir * wii300_readdir( FILE * );
void wii300_readdirtype( FILE *, struct wii300dir * );
void wii300_readdirdesc( FILE *, struct wii300dir * );
void wii300_readdirplaceintree( FILE *, struct wii300dir * );
void wii300_readdirname( FILE *, struct wii300dir * );
void wii300_readfilesindir( FILE *, struct wii300dir * );
void wii300_readdircreationdate( FILE *, struct wii300dir * );
void wii300_readdirdescription( FILE *, struct wii300dir * );
void wii300_freedir( struct wii300dir * );

time_t wii300_readdate( FILE * );
unsigned int wii300_getplaintextlength( FILE *, unsigned int );
void wii300_skipplaintext( FILE *, unsigned char );
unsigned int wii300_addnormal( unsigned int, unsigned int );
unsigned long wii300_addsize( unsigned int, unsigned int, unsigned int, unsigned );

#endif
